package cn.com.qmth.examcloud.core.oe.task.service;

import cn.com.qmth.examcloud.core.oe.admin.api.bean.*;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.bean.StuExamQuestionBean;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamFaceLivenessVerifiesReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordPaperStructReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordQuestionsReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetFaceBiopsyReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamFaceLivenessVerifiesResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordPaperStructResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordQuestionsResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetFaceBiopsyResp;
import cn.com.qmth.examcloud.core.oe.student.dao.*;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.*;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Component
public class CommonService {

    private static final Logger log = LoggerFactory.getLogger(CommonService.class);

    @Autowired
    private ExamCaptureRepo examCaptureRepo;

    @Autowired
    private ExamFaceLiveVerifyRepo examFaceLiveVerifyRepo;

    @Autowired
    private ExamSyncCaptureRepo examSyncCaptureRepo;

    @Autowired
    private ExamContinuedRecordRepo examContinuedRecordRepo;

    @Autowired
    private ExamProcessRecordRepo examProcessRecordRepo;

    @Autowired
    private ExamRecordDataCloudService examRecordDataCloudService;

    public ExamRecordDataBean copyExamRecordDataFrom(ExamRecordData examRecordData) {
        ExamRecordDataBean data = new ExamRecordDataBean();

        data.setId(examRecordData.getId());
        data.setRootOrgId(examRecordData.getRootOrgId());
        data.setOrgId(examRecordData.getOrgId());
        data.setCourseId(examRecordData.getCourseId());
        data.setStudentId(examRecordData.getStudentId());
        data.setExamStudentId(examRecordData.getExamStudentId());

        data.setExamId(examRecordData.getExamId());
        data.setExamType(examRecordData.getExamType() != null ? examRecordData.getExamType().toString() : null);
        data.setExamStageId(examRecordData.getExamStageId());
        data.setExamStageOrder(examRecordData.getExamStageOrder());
        data.setBasePaperId(examRecordData.getBasePaperId());
        data.setPaperType(examRecordData.getPaperType());

        data.setStartTime(examRecordData.getStartTime());
        data.setEndTime(examRecordData.getEndTime());
        data.setEnterExamTime(examRecordData.getEnterExamTime());
        data.setCleanTime(examRecordData.getCleanTime());
        data.setLastActiveTime(examRecordData.getLastActiveTime());
        data.setUsedExamTime(examRecordData.getUsedExamTime());

        data.setContinuedCount(examRecordData.getContinuedCount());
        data.setContinuedTime(examRecordData.getContinuedTime());
        data.setContinued(examRecordData.getIsContinued() != null ? examRecordData.getIsContinued() : false);
        data.setExceed(examRecordData.getIsExceed() != null ? examRecordData.getIsExceed() : false);

        data.setExamRecordStatus(examRecordData.getExamRecordStatus() != null ? examRecordData.getExamRecordStatus().toString() : null);
        data.setWarn(examRecordData.getIsWarn() != null ? examRecordData.getIsWarn() : false);
        data.setAudit(examRecordData.getIsAudit() != null ? examRecordData.getIsAudit() : false);
        data.setIllegality(examRecordData.getIsIllegality() != null ? examRecordData.getIsIllegality() : false);

        data.setFaceVerifyResult(examRecordData.getFaceVerifyResult() != null ? examRecordData.getFaceVerifyResult().toString() : null);
        data.setBaiduFaceLivenessSuccessPercent(examRecordData.getBaiduFaceLivenessSuccessPercent());
        data.setFaceTotalCount(examRecordData.getFaceTotalCount());
        data.setFaceSuccessCount(examRecordData.getFaceSuccessCount());
        data.setFaceFailedCount(examRecordData.getFaceFailedCount());
        data.setFaceStrangerCount(examRecordData.getFaceStrangerCount());
        data.setFaceSuccessPercent(examRecordData.getFaceSuccessPercent());

        data.setTotalScore(examRecordData.getTotalScore());
        data.setObjectiveScore(examRecordData.getObjectiveScore());
        data.setObjectiveAccuracy(examRecordData.getObjectiveAccuracy());
        data.setSubjectiveScore(examRecordData.getSubjectiveScore());
        data.setSuccPercent(examRecordData.getSuccPercent());
        data.setAllObjectivePaper(examRecordData.getIsAllObjectivePaper());

        data.setSwitchScreenCount(examRecordData.getSwitchScreenCount());
        data.setExceedMaxSwitchScreenCount(examRecordData.getExceedMaxSwitchScreenCount());
        return data;
    }

    public ExamRecordPaperStructBean getExamRecordPaperStruct(Long examRecordDataId) {
        GetExamRecordPaperStructReq req = new GetExamRecordPaperStructReq();
        req.setExamRecordDataId(examRecordDataId);
        GetExamRecordPaperStructResp resp = examRecordDataCloudService.getExamRecordPaperStruct(req);

        ExamRecordPaperStructBean bean = new ExamRecordPaperStructBean();
        bean.setId(resp.getId());
        bean.setDefaultPaper(resp.getDefaultPaper());
        return bean;
    }

    public ExamRecordQuestionsBean getExamRecordQuestions(Long examRecordDataId) {
        GetExamRecordQuestionsReq req = new GetExamRecordQuestionsReq();
        req.setExamRecordDataId(examRecordDataId);
        GetExamRecordQuestionsResp resp = examRecordDataCloudService.getExamRecordQuestions(req);

        ExamRecordQuestionsBean bean = new ExamRecordQuestionsBean();
        bean.setCreationTime(resp.getCreationTime());
        bean.setExamRecordDataId(resp.getExamRecordDataId());

        List<ExamQuestionBean> examQuestionBeanList = new ArrayList<>();
        for (StuExamQuestionBean stuExamQuestionBean : resp.getExamQuestions()) {
            ExamQuestionBean examQuestionBean = new ExamQuestionBean();
            examQuestionBean.setExamRecordDataId(stuExamQuestionBean.getExamRecordDataId());
            examQuestionBean.setMainNumber(stuExamQuestionBean.getMainNumber());
            examQuestionBean.setQuestionId(stuExamQuestionBean.getQuestionId());
            examQuestionBean.setOrder(stuExamQuestionBean.getOrder());
            examQuestionBean.setQuestionScore(stuExamQuestionBean.getQuestionScore());
            examQuestionBean.setQuestionType(stuExamQuestionBean.getQuestionType());
            examQuestionBean.setCorrectAnswer(stuExamQuestionBean.getCorrectAnswer());
            examQuestionBean.setStudentAnswer(stuExamQuestionBean.getStudentAnswer());
            examQuestionBean.setStudentScore(stuExamQuestionBean.getStudentScore());
            examQuestionBean.setAnswer(stuExamQuestionBean.getAnswer());
            examQuestionBean.setSign(stuExamQuestionBean.getSign());
            examQuestionBean.setOptionPermutation(stuExamQuestionBean.getOptionPermutation());
            examQuestionBean.setAudioPlayTimes(stuExamQuestionBean.getAudioPlayTimes());

            if (null != stuExamQuestionBean.getAnswerType()) {
                examQuestionBean.setAnswerType(stuExamQuestionBean.getAnswerType().name());
            }

            examQuestionBeanList.add(examQuestionBean);
        }

        bean.setExamQuestionBeans(examQuestionBeanList);

        return bean;
    }

    public List<ExamContinuedRecordBean> getExamContinuedRecords(Long examRecordDataId) {
        List<ExamContinuedRecordEntity> entityList = examContinuedRecordRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entityList || entityList.isEmpty()) {
            return new ArrayList<>();
        }

        List<ExamContinuedRecordBean> resultList = new ArrayList<>();
        for (ExamContinuedRecordEntity entity : entityList) {
            ExamContinuedRecordBean bean = new ExamContinuedRecordBean();
            bean.setId(entity.getId());
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setContinuedTime(entity.getContinuedTime());
            bean.setStartTime(entity.getStartTime());
            bean.setUsedExamTime(entity.getUsedExamTime());

            resultList.add(bean);
        }

        return resultList;
    }

    public List<ExamProcessRecordBean> getExamProcessRecords(Long examRecordDataId) {
        List<ExamProcessRecordEntity> entityList = examProcessRecordRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entityList || entityList.isEmpty()) {
            return new ArrayList<>();
        }

        List<ExamProcessRecordBean> resultList = new ArrayList<>();
        for (ExamProcessRecordEntity entity : entityList) {
            ExamProcessRecordBean bean = new ExamProcessRecordBean();
            bean.setId(entity.getId());
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setProcessName(entity.getProcessName());
            bean.setRecordTime(entity.getRecordTime());
            bean.setSourceIp(entity.getSourceIp());

            resultList.add(bean);
        }

        return resultList;
    }

    public ExamSyncCaptureBean getExamSyncCapture(Long examRecordDataId) {
        ExamSyncCaptureEntity entity = examSyncCaptureRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entity) {
            return null;
        }

        ExamSyncCaptureBean bean = new ExamSyncCaptureBean();
        bean.setId(entity.getId());
        bean.setExamRecordDataId(entity.getExamRecordDataId());
        bean.setFileUrl(entity.getFileUrl());
        bean.setFileName(entity.getFileName());
        bean.setPass(entity.getIsPass());
        bean.setFaceCompareResult(entity.getFaceCompareResult());
        bean.setStranger(entity.getIsStranger());
        bean.setLandmark(entity.getLandmark());
        bean.setFacelivenessResult(entity.getFacelivenessResult());
        bean.setUsedTime(entity.getUsedTime());
        bean.setProcessTime(entity.getProcessTime());
        bean.setHasVirtualCamera(entity.getHasVirtualCamera());
        bean.setCameraInfos(entity.getCameraInfos());
        bean.setExtMsg(entity.getExtMsg());

        return bean;
    }

    public List<ExamCaptureBean> getExamCaptures(Long examRecordDataId) {
        List<ExamCaptureEntity> entityList = examCaptureRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entityList || entityList.isEmpty()) {
            return new ArrayList<>();
        }

        List<ExamCaptureBean> resultList = new ArrayList<>();
        for (ExamCaptureEntity entity : entityList) {
            ExamCaptureBean bean = new ExamCaptureBean();

            bean.setId(entity.getId());
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setFileUrl(entity.getFileUrl());
            bean.setFileName(entity.getFileName());
            bean.setPass(entity.getIsPass());
            bean.setFaceCompareResult(entity.getFaceCompareResult());
            bean.setStranger(entity.getIsStranger());
            bean.setLandmark(entity.getLandmark());
            bean.setFacelivenessResult(entity.getFacelivenessResult());
            bean.setUsedTime(entity.getUsedTime());
            bean.setProcessTime(entity.getProcessTime());
            bean.setHasVirtualCamera(entity.getHasVirtualCamera());
            bean.setCameraInfos(entity.getCameraInfos());
            bean.setExtMsg(entity.getExtMsg());

            resultList.add(bean);
        }

        return resultList;
    }

    public List<ExamFaceLivenessVerifyBean> getExamFaceLivenessVerifies(Long examRecordDataId) {
        GetExamFaceLivenessVerifiesReq req = new GetExamFaceLivenessVerifiesReq();
        req.setExamRecordDataId(examRecordDataId);

        GetExamFaceLivenessVerifiesResp resp = examRecordDataCloudService.getExamFaceLivenessVerifies(req);
        if (null == resp.getExamFaceLivenessVerifis() || resp.getExamFaceLivenessVerifis().isEmpty()) {
            return new ArrayList<>();
        }

        List<ExamFaceLivenessVerifyBean> resultList = new ArrayList<>();
        for (cn.com.qmth.examcloud.core.oe.student.api.bean.ExamFaceLivenessVerifyBean eflvb : resp.getExamFaceLivenessVerifis()) {
            ExamFaceLivenessVerifyBean bean = new ExamFaceLivenessVerifyBean();
            bean.setId(eflvb.getId());
            bean.setExamRecordDataId(eflvb.getExamRecordDataId());
            bean.setStartTime(eflvb.getStartTime());
            bean.setUsedTime(eflvb.getUsedTime());
            bean.setResultJson(eflvb.getResultJson());
            bean.setVerifyResult(eflvb.getVerifyResult());
            bean.setBizId(eflvb.getBizId());
            bean.setIsError(eflvb.getIsError());
            bean.setErrorMsg(eflvb.getErrorMsg());
            bean.setOperateNum(eflvb.getOperateNum());

            resultList.add(bean);
        }

        return resultList;
    }

    public FaceBiopsyBean getFaceBiopsy(Long examRecordDataId) {
        GetFaceBiopsyReq req = new GetFaceBiopsyReq();
        req.setExamRecordDataId(examRecordDataId);

        GetFaceBiopsyResp resp = examRecordDataCloudService.getFaceBiopsy(req);
        if (null == resp.getFaceBiopsyBean()) {
            return null;
        }

        cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyBean faceBiopsyBean = resp.getFaceBiopsyBean();
        FaceBiopsyBean bean = new FaceBiopsyBean();
        bean.setErrorMsg(faceBiopsyBean.getErrorMsg());
        bean.setExamRecordDataId(faceBiopsyBean.getExamRecordDataId());
        bean.setResult(faceBiopsyBean.getResult());
        bean.setRootOrgId(faceBiopsyBean.getRootOrgId());
        bean.setVerifiedTimes(faceBiopsyBean.getVerifiedTimes());

        List<FaceBiopsyItemBean> faceBiopsyItemList = new ArrayList<>();
        for (cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyItemBean faceBiopsyItemBean : faceBiopsyBean.getFaceBiopsyItems()) {
            FaceBiopsyItemBean itemBean = new FaceBiopsyItemBean();

            itemBean.setCompleted(faceBiopsyItemBean.getCompleted());
            itemBean.setErrorMsg(faceBiopsyItemBean.getErrorMsg());
            itemBean.setExamRecordDataId(faceBiopsyItemBean.getExamRecordDataId());
            itemBean.setFaceBiopsyId(faceBiopsyItemBean.getFaceBiopsyId());
            itemBean.setFaceBiopsyType(faceBiopsyItemBean.getFaceBiopsyType());
            itemBean.setInFreezeTime(faceBiopsyItemBean.getInFreezeTime());
            itemBean.setResult(faceBiopsyItemBean.getResult());

            List<FaceBiopsyItemStepBean> itemStepList = new ArrayList<>();
            for (cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyItemStepBean faceBiopsyItemStepBean : faceBiopsyItemBean.getFaceBiopsyItemSteps()) {
                FaceBiopsyItemStepBean itemStepBean = new FaceBiopsyItemStepBean();
                itemStepBean.setAction(faceBiopsyItemStepBean.getAction());
                itemStepBean.setActionStay(faceBiopsyItemStepBean.getActionStay());
                itemStepBean.setErrorMsg(faceBiopsyItemStepBean.getErrorMsg());
                itemStepBean.setExamRecordDataId(faceBiopsyItemStepBean.getExamRecordDataId());
                itemStepBean.setFaceBiopsyItemId(faceBiopsyItemStepBean.getFaceBiopsyItemId());
                itemStepBean.setResourceRelativePath(faceBiopsyItemStepBean.getResourceRelativePath());
                itemStepBean.setResourceType(faceBiopsyItemStepBean.getResourceType());
                itemStepBean.setResult(faceBiopsyItemStepBean.getResult());

                itemStepList.add(itemStepBean);
            }

            itemBean.setFaceBiopsyItemSteps(itemStepList);
        }

        bean.setFaceBiopsyItems(faceBiopsyItemList);

        return bean;
    }

    public List<FaceLiveVerifyBean> getFaceLiveVerifyRecords(Long examRecordDataId) {
        List<ExamFaceLiveVerifyEntity> entities = examFaceLiveVerifyRepo.findByExamRecordDataIdAndFinished(examRecordDataId, true);
        if (CollectionUtils.isEmpty(entities)) {
            return null;
        }

        List<FaceLiveVerifyBean> faceLiveVerifyRecords = new ArrayList<>();
        for (ExamFaceLiveVerifyEntity entity : entities) {
            FaceLiveVerifyBean bean = new FaceLiveVerifyBean();
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setStatus(entity.getStatus().name());
            bean.setFaceCount(entity.getFaceCount());
            bean.setSimilarity(entity.getSimilarity());
            bean.setRealness(entity.getRealness());
            bean.setErrorMsg(entity.getErrorMsg());
            bean.setProcessTime(entity.getProcessTime());
            bean.setActions(entity.getActions());
            bean.setCreationTime(entity.getCreationTime());
            bean.setUpdateTime(entity.getUpdateTime());
            faceLiveVerifyRecords.add(bean);
        }

        return faceLiveVerifyRecords;
    }

}
