package cn.com.qmth.examcloud.core.oe.task.service.bean;

import cn.com.qmth.examcloud.api.commons.exchange.JsonSerializable;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.HashSet;
import java.util.Set;

public class RecordDataSyncRule implements JsonSerializable {

    private static final long serialVersionUID = 802462624028077528L;

    private Set<Long> allowOrgList;

    private Set<Long> limitOrgList;

    private Boolean allowOrgAll;

    public RecordDataSyncRule(String recordDataSyncAllowOrgList, String recordDataSyncLimitOrgList) {
        /*
            规则：
            allowOrgList 和 limitOrgList 若配置值，则按值处理
            allowOrgList 和 limitOrgList 若都未配置值，则按允许所有处理
         */

        Set<Long> allowOrgList = new HashSet<>();
        if (StringUtils.isNotEmpty(recordDataSyncAllowOrgList)) {
            String[] values = StringUtils.split(recordDataSyncAllowOrgList, ",");
            for (String value : values) {
                if (StringUtils.isBlank(value)) {
                    continue;
                }

                allowOrgList.add(Long.parseLong(value.trim()));
            }
        }

        Set<Long> limitOrgList = new HashSet<>();
        if (StringUtils.isNotEmpty(recordDataSyncLimitOrgList)) {
            String[] values = StringUtils.split(recordDataSyncLimitOrgList, ",");
            for (String value : values) {
                if (StringUtils.isBlank(value)) {
                    continue;
                }

                limitOrgList.add(Long.parseLong(value.trim()));
            }
        }

        this.allowOrgList = allowOrgList;
        this.limitOrgList = limitOrgList;

        if (CollectionUtils.isEmpty(allowOrgList) && CollectionUtils.isEmpty(limitOrgList)) {
            this.allowOrgAll = true;
        } else {
            this.allowOrgAll = false;
        }
    }

    public boolean isAllowOrg(Long orgId) {
        if (this.allowOrgList == null) {
            return false;
        }
        return this.allowOrgList.contains(orgId);
    }

    public boolean isLimitOrg(Long orgId) {
        if (this.limitOrgList == null) {
            return false;
        }
        return this.limitOrgList.contains(orgId);
    }

    public Set<Long> getAllowOrgList() {
        return allowOrgList;
    }

    public void setAllowOrgList(Set<Long> allowOrgList) {
        this.allowOrgList = allowOrgList;
    }

    public Set<Long> getLimitOrgList() {
        return limitOrgList;
    }

    public void setLimitOrgList(Set<Long> limitOrgList) {
        this.limitOrgList = limitOrgList;
    }

    public boolean isAllowOrgAll() {
        return allowOrgAll;
    }

    public void setAllowOrgAll(boolean allowOrgAll) {
        this.allowOrgAll = allowOrgAll;
    }

    @Override
    public String toString() {
        return "RecordDataSyncRule{" +
                "allowOrgList=" + allowOrgList +
                ", limitOrgList=" + limitOrgList +
                ", allowOrgAll=" + allowOrgAll +
                '}';
    }

}
