package cn.com.qmth.examcloud.core.oe.task.service.job;

import cn.com.qmth.examcloud.core.oe.student.dao.ExamCaptureQueueRepo;
import cn.com.qmth.examcloud.core.oe.task.service.ExamCaptureQueueService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.web.exception.SequenceLockException;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 处理考试过程中抓拍照片比对任务
 */
@Component
public class FaceVerifyJobHandler {

    private static final Logger log = LoggerFactory.getLogger(FaceVerifyJobHandler.class);

    @Autowired
    private ExamCaptureQueueRepo examCaptureQueueRepo;

    @Autowired
    private ExamCaptureQueueService examCaptureQueueService;

    public void run(int shardTotal, int shardIndex, String jobParam) throws Exception {
        int batchSize = 100, maxErrorNum = 10;
        List<Long> todoExamRecordDataIds = examCaptureQueueRepo.findQueuesGroupByExamRecordDataId(shardTotal, shardIndex, batchSize, maxErrorNum);

        log.warn("分片任务_{}_{}  curTodoExamRecordDataIds_size:{}", shardTotal, shardIndex, todoExamRecordDataIds.size());
        if (CollectionUtils.isEmpty(todoExamRecordDataIds)) {
            return;
        }

        for (Long examRecordDataId : todoExamRecordDataIds) {
            final String lockKey = Constants.FACE_COMPARE_LOCK_PREFIX + examRecordDataId;
            try {
                SequenceLockHelper.getLockSimple(lockKey);

                // 处理未比对的抓拍照片
                examCaptureQueueService.handlerExamCaptureQueuesByExamRecordDataId(examRecordDataId);
            } catch (Exception e) {
                if (e instanceof InterruptedException) {
                    // 若线程终止，则抛出交由任务调度中心处理
                    log.warn("当前人脸比对任务线程被终止！examRecordDataId:{}, error:{}", examRecordDataId, e.getMessage());
                    throw e;
                } else if (e instanceof SequenceLockException) {
                    // 若锁问题，下次会继续执行
                    log.warn("当前人脸比对任务获取锁失败！examRecordDataId:{}, redisKey:{}", examRecordDataId, lockKey);
                } else {
                    // 若异常，下次会继续执行（需要排查原因）
                    log.error("当前人脸比对任务处理失败！examRecordDataId:{}, error:{}", examRecordDataId, e.getMessage());
                }
            } finally {
                SequenceLockHelper.releaseLockSimple(lockKey);
            }
        }
    }

}
