package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description 交卷后续处理执行器
 * @Author lideyin
 * @Date 2019/12/17 16:39
 * @Version 1.0
 */
@Deprecated
@Component
public class AfterHandInExamExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private ExamRecordDataService examRecordDataService;

    private static Long DEFAULT_MAX_PROCESS_SECONDS = 30L;

    private static final Logger LOG = LoggerFactory.getLogger(AfterHandInExamExecutor.class);

    /**
     * 执行
     *
     * @param key
     * @param examRecordData
     * @param outList
     * @param removable
     * @param context
     * @throws Exception
     * @author WANGWEI
     */
    @Override
    public void execute(Long key, ExamRecordData examRecordData,
                        List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {

        String sequenceLockKey = Constants.EXAM_CONTROL_LOCK_PREFIX + examRecordData.getStudentId();

        long st = System.currentTimeMillis();

        try {
            this.debugLog("enter executor...", examRecordData.getId());

            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            this.debugLog("get locker success...", examRecordData.getId());

            //针对已交卷的数据进行交卷后续处理
            if (examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_HAND_IN ||
                    examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_AUTO_HAND_IN) {

                this.debugLog("prepare to execute after hand in ...", examRecordData.getId());

                SysPropertyCacheBean maxProcessSecondsProperty = CacheHelper.getSysProperty("oe.task.maxProcessSeconds");
                //本节点最大处理时长
                Long maxProcessSeconds;
                if (maxProcessSecondsProperty.getHasValue()) {
                    maxProcessSeconds = Long.valueOf(maxProcessSecondsProperty.getValue().toString());
                } else {
                    maxProcessSeconds = DEFAULT_MAX_PROCESS_SECONDS;
                }

                //交卷时间戳
                Long handInTime = (examRecordData.getEndTime() == null
                        ? examRecordData.getCleanTime()
                        : examRecordData.getEndTime()).getTime();
                Long times = System.currentTimeMillis() - handInTime;

                //如果交卷后超过指定时长内仍未处理完成，则交给下一节点进行处理
                if (times > maxProcessSeconds * 1000) {
                    outList.add(new KeyValuePair<>(key, examRecordData));

                    this.debugLog("not complete in time and will retry in next node...", examRecordData.getId());

                    return;
                }

                examRecordData = examRecordDataService.processAfterHandInExam(examRecordData.getId());

                outList.add(new KeyValuePair<>(key, examRecordData));

                this.debugLog("after hand in success..", examRecordData.getId());

                return;
            }

            //其它状态数据，直接交给下一步
            outList.add(new KeyValuePair<>(key, examRecordData));

            this.debugLog("current status is '" + examRecordData.getExamRecordStatus().name() +
                    "'.do nothing and go to the next node...", examRecordData.getId());
        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
            this.debugLog("T999 HandInExamExecutor合计耗时:" +
                    (System.currentTimeMillis() - st) + " ms", examRecordData.getId());
        }
    }

    private void debugLog(String msg, Long examRecordDataId) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[AFTER-HAND-IN-EXAM-EXECUTOR-" + examRecordDataId + "]：" + msg);
        }
    }

}
