package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.task.service.ExamingSessionService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import cn.com.qmth.examcloud.support.examing.ExamBoss;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @Description 清理考试记录缓存
 * @Author lideyin
 * @Date 2019/12/24 18:07
 * @Version 1.0
 */
@Deprecated
@Component
public class ClearExamDataCacheExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private RedisClient redisClient;

    private static final Logger LOG = LoggerFactory.getLogger(ClearExamDataCacheExecutor.class);

    @Autowired
    private ExamingSessionService examingSessionService;

    /**
     * 执行
     *
     * @param key
     * @param examRecordData
     * @param outList
     * @param removable
     * @param context
     * @throws Exception
     */
    @Override
    public void execute(Long key, ExamRecordData examRecordData,
                        List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {

        String sequenceLockKey = Constants.EXAM_CONTROL_LOCK_PREFIX + examRecordData.getStudentId();

        long st = System.currentTimeMillis();

        try {
            Long examRecordDataId = examRecordData.getId();
            debugLog("enter executor...", examRecordDataId);

            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            debugLog("get locker success...", examRecordDataId);

            //只有已同步成功的数据，才执行清理操作
            if (SyncStatus.SYNCED == examRecordData.getSyncStatus()) {

                //清除考试次数相关缓存
                clearExamBoss(examRecordData.getExamStudentId(), examRecordDataId);

                //清除学生最后活动时间缓存
                redisClient.delete(RedisKeyHelper.getBuilder().examingActiveTimeKey(examRecordDataId));

                //清除心跳缓存
                redisClient.delete(RedisKeyHelper.getBuilder().examingHeartbeatKey(examRecordDataId));

                //清除考试会话缓存
                ExamingSession examingSession = examingSessionService.getExamingSession(examRecordData.getStudentId());
                if (null != examingSession && examingSession.getExamRecordDataId().equals(examRecordDataId)) {
                    String sessionKey = RedisKeyHelper.getBuilder().examingSessionKey(examRecordData.getStudentId());
                    redisClient.delete(sessionKey);
                    this.infoLog(String.format("清理交卷未删除的会话:%s", sessionKey), examRecordDataId);
                }

                //清除文件作答和普通作答记录缓存
                Integer quesCount = examRecordData.getQuestionCount();
                for (int i = 1; i <= quesCount; i++) {
                    redisClient.delete(RedisKeyHelper.getBuilder().studentFileAnswerKey(examRecordDataId, i));
                    redisClient.delete(RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, i));
                }

                //清除网考试卷结构
                redisClient.delete(RedisKeyHelper.getBuilder().studentPaperKey(examRecordDataId));

                //清除考试记录缓存(设置缓存的过期时间)
                Long defaultExpiredSeconds = PropertyHolder.getLong("oe.student.cache.expiredSeconds", 180);//缓存默认过期时间
                redisClient.expire(RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId),
                        defaultExpiredSeconds, TimeUnit.SECONDS);

                redisClient.delete(Constants.OE_DISCIPLINE_ILLEGAL_DATA + examRecordDataId);
                redisClient.delete(Constants.OE_DISCIPLINE_ILLEGAL_CLIENT + examRecordDataId);

                this.debugLog("all is over.", examRecordDataId);

                return;
            }

            this.debugLog("current status is '" + examRecordData.getExamRecordStatus().name() +
                    "'.may be redundant data.", examRecordDataId);

        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
            this.debugLog("T999 ClearExamDataCacheExecutor合计耗时:" +
                    (System.currentTimeMillis() - st) + " ms", examRecordData.getId());
        }
    }

    private void clearExamBoss(Long examStudentId, Long examRecordDataId) {
        String examBossKey = RedisKeyHelper.getBuilder().examBossKey(examStudentId);
        ExamBoss examBoss = redisClient.get(examBossKey, ExamBoss.class);
        if (null != examBoss) {
            //如果开考次数==考试完结次数，则删除考试基础信息缓存
            if (examBoss.getStartCount() == examBoss.getEndCount()) {
                redisClient.delete(examBossKey);
            }
            //如果examBoss未清除，则清除当前的考试记录id
            else {
                List<Long> examRecordDataIds = examBoss.getExamRecordDataIds();
                if (null != examRecordDataIds && !examRecordDataIds.isEmpty()) {
                    examRecordDataIds.removeIf(p -> examRecordDataId.equals(p));
                    examBoss.setExamRecordDataIds(examRecordDataIds);
                    redisClient.set(examBossKey, examBoss);
                }
            }
        }
    }

    private void debugLog(String msg, Long examRecordDataId) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[CLEAR-EXAM-DATA-CACHE-EXECUTOR-" + examRecordDataId + "]：" + msg);
        }
    }

    private void infoLog(String msg, Long examRecordDataId) {
        if (LOG.isInfoEnabled()) {
            LOG.info("[CLEAR-EXAM-DATA-CACHE-EXECUTOR-" + examRecordDataId + "]：" + msg);
        }
    }

}
