package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordDataIdsReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdateExamRecordDataBatchNumReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdateExamRecordStatusReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordDataIdsResp;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.task.service.bean.RecordDataSyncRule;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description 获取待处理数据
 * @Author xt
 * @Date 2019/12/17 16:39
 * @Version 1.0
 */
@Deprecated
@Component
public class DataGainExamExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    private static final Logger LOG = LoggerFactory.getLogger(DataGainExamExecutor.class);

    private final static Integer batchSize = 200;

    private final static Long batchNum = new Date().getTime();

    @Autowired
    private ExamRecordDataCloudService examRecordDataCloudService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Value("${examcloud.record.data.sync.allowOrgList}")
    private String recordDataSyncAllowOrgList;

    @Value("${examcloud.record.data.sync.limitOrgList}")
    private String recordDataSyncLimitOrgList;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public void execute(Long key, ExamRecordData value, List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {
        RecordDataSyncRule syncRule = new RecordDataSyncRule(recordDataSyncAllowOrgList, recordDataSyncLimitOrgList);
        LOG.warn(syncRule.toString());

        LOG.info("开始获取数据");
        Date start = new Date();
        // 获取考试信息id
        Long startId = 0l;
        GetExamRecordDataIdsReq req = new GetExamRecordDataIdsReq();
        req.setBatchNum(batchNum);
        req.setSize(batchSize);

        for (; ; ) {
            req.setStartId(startId);
            GetExamRecordDataIdsResp res = null;

            try {
                res = examRecordDataCloudService.getExamRecordDataIds(req);
            } catch (Exception e) {
                Date end = new Date();
                long times = end.getTime() - start.getTime();
                LOG.error("获取数据库中考试信息出错 startId:" + startId + " 获取数据条数：" + outList.size() + " 耗时：" + times + "ms", e);
                return;
            }

            List<Long> ids = res.getExamRecordDataIds();
            if (ids == null || ids.size() == 0) {
                Date end = new Date();
                long times = end.getTime() - start.getTime();
                LOG.info("获取数据条数：" + outList.size() + " 耗时：" + times + "ms");
                return;
            }

            List<KeyValuePair<Long, ExamRecordData>> tempList = new ArrayList<>();
            List<Long> newExamRecordDataIds = new ArrayList<>();

            for (Long id : ids) {
                // 根据id获取考试信息缓存
                ExamRecordData examRecordData = examRecordDataService.getExamRecordDataCache(id);

                if (examRecordData == null) {
                    LOG.error("获取Redis中考试信息为空 examRecordDataId:" + id);
                    examRecordData = this.queryExamRecordData(id);

                    if (examRecordData != null) {

                        if (syncRule.getAllowOrgList().isEmpty()) {
                            // 未配置“允许机构列表”值时，默认执行所有机构，但要跳过“限制机构列表”中的机构
                            if (syncRule.isLimitOrg(examRecordData.getRootOrgId())) {
                                continue;
                            }

                            updateExamRecordStatusError(id);
                        } else {
                            // 配置了“允许机构列表”值时，只执行“允许机构列表”范围内的机构
                            if (syncRule.isAllowOrg(examRecordData.getRootOrgId())) {
                                updateExamRecordStatusError(id);
                            }
                        }

                    }
                } else {

                    if (syncRule.getAllowOrgList().isEmpty()) {
                        // 未配置“允许机构列表”值时，默认执行所有机构，但要跳过“限制机构列表”中的机构
                        if (syncRule.isLimitOrg(examRecordData.getRootOrgId())) {
                            continue;
                        }

                        newExamRecordDataIds.add(id);
                        tempList.add(new KeyValuePair<>(id, examRecordData));
                    } else {
                        // 配置了“允许机构列表”值时，只执行“允许机构列表”范围内的机构
                        if (syncRule.isAllowOrg(examRecordData.getRootOrgId())) {
                            newExamRecordDataIds.add(id);
                            tempList.add(new KeyValuePair<>(id, examRecordData));
                        }
                    }

                }
            }

            // 修改已获取过的考试信息batchNum
            UpdateExamRecordDataBatchNumReq ureq = new UpdateExamRecordDataBatchNumReq();
            ureq.setBatchNum(batchNum);
            ureq.setIds(newExamRecordDataIds);
            try {
                examRecordDataCloudService.updateExamRecordDataBatchNum(ureq);
            } catch (Exception e) {
                Date end = new Date();
                long times = end.getTime() - start.getTime();
                LOG.error("修改考试记录batchNum出错 startId:" + startId + " 获取数据条数：" + outList.size() + " 耗时：" + times + "ms", e);
                return;
            }

            outList.addAll(tempList);
            startId = ids.get(ids.size() - 1);
        }
    }

    private ExamRecordData queryExamRecordData(Long id) {
        String sql = String.format("select id,root_org_id,exam_record_status,sync_status from ec_oes_exam_record_data where id = %s", id);
        try {
            return jdbcTemplate.queryForObject(sql, new BeanPropertyRowMapper<>(ExamRecordData.class));
        } catch (Exception e) {
            // ignore
            return null;
        }
    }

    private void updateExamRecordStatusError(Long id) {
        try {
            UpdateExamRecordStatusReq ureq = new UpdateExamRecordStatusReq();
            ureq.setExamRecordStatus(ExamRecordStatus.EXAM_ERROR.name());
            ureq.setId(id);
            examRecordDataCloudService.updateExamRecordStatus(ureq);
        } catch (Exception e) {
            LOG.error("修改考试记录状态失败 examRecordDataId:" + id, e);
        }
    }

}
