package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.request.HandInExamReq;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.task.service.ExamingSessionService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.enums.HandInExamType;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.examing.ExamingActivityTime;
import cn.com.qmth.examcloud.support.examing.ExamingHeartbeat;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @Description 自动交卷执行器
 * @Author lideyin
 * @Date 2019/12/17 16:39
 * @Version 1.0
 */
@Deprecated
@Component
public class HandInExamExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private ExamingSessionService examingSessionService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private ExamRecordDataCloudService examRecordDataCloudService;

    private static final Logger LOG = LoggerFactory.getLogger(HandInExamExecutor.class);

    /**
     * 执行
     *
     * @param key
     * @param uncertainExamRecordData 不确定的考试记录，数据有可能已变更
     * @param outList
     * @param removable
     * @param context
     * @throws Exception
     * @author WANGWEI
     */
    @Override
    public void execute(Long key, ExamRecordData uncertainExamRecordData,
                        List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {

        String sequenceLockKey = Constants.EXAM_CONTROL_LOCK_PREFIX + uncertainExamRecordData.getStudentId();

        try {

            this.debugLog("enter executor...", uncertainExamRecordData.getId());

            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            this.debugLog("get locker success...", uncertainExamRecordData.getId());

            //获取最新的考试记录状态
            ExamRecordData examRecordData = examRecordDataService.getExamRecordDataCache(uncertainExamRecordData.getId());

            Long examRecordDataId = examRecordData.getId();

            //处理正在进行中的考试
            if (examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_ING) {

                ExamingSession examingSession = examingSessionService.getExamingSession(examRecordData.getStudentId());

                if (null == examingSession) {
                    this.debugLog("examingSession is null...", examRecordDataId);
                } else {
                    this.debugLog("examingSession is not null and the json is..." + JsonUtil.toJson(examingSession),
                            examRecordDataId);
                }

                // 如果考试会话不存在/超过断点续考时间，自动交卷
                if (null == examingSession || isOverBreakpointTime(examingSession)) {

                    try {

                        this.debugLog("prepare to auto hand in...", examRecordDataId);

                        //自动交卷
                        HandInExamReq handInExamReq = new HandInExamReq();
                        handInExamReq.setExamRecordDataId(examRecordDataId);
                        handInExamReq.setHandInExamType(HandInExamType.AUTO);
                        examRecordDataCloudService.handInExam(handInExamReq);

                        //更改内存中的交卷状态
                        examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_AUTO_HAND_IN);
                        examRecordData.setCleanTime(new Date());

                        //更新考试记录缓存
                        examRecordDataService.saveExamRecordDataCache(examRecordDataId, examRecordData);

                        // 删除考试会话
                        if (null != examingSession) {
                            examingSessionService.deleteExamingSession(examRecordData.getStudentId());
                        }

                        outList.add(new KeyValuePair<>(key, examRecordData));

                        this.debugLog("auto hand in success...", examRecordDataId);

                        return;

                    } catch (Exception e) {
                        //回滚自动交卷操作
                        examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
                        examRecordData.setCleanTime(null);
                        examRecordDataService.saveExamRecordDataCache(examRecordDataId, examRecordData);

                        outList.clear();
                        removable.set(false);

                        this.errorLog("auto hand in occurs error,to be retry...", e, examRecordDataId);

                        throw new StatusException("300101", "自动交卷出现异常：" + e.getMessage());
                    }
                }

                //如果不需要自动交卷，则需要下次轮循，继续处理
                removable.set(false);
                outList.clear();

                this.debugLog("don't need auto hand in.to be retry...", examRecordDataId);

                return;
            }

            //其它状态的数据，直接交给下一个节点处理
            outList.add(new KeyValuePair<>(key, examRecordData));

            this.debugLog("current status is '" + examRecordData.getExamRecordStatus().name() +
                    "'.do nothing and go to the next node...", examRecordDataId);
        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
        }
    }

    /**
     * 是否超过考试时长
     *
     * @param examingSession
     * @return
     */
    private boolean isOverExamTime(ExamingSession examingSession) {
        String examingHeartbeatKey = RedisKeyHelper.getBuilder()
                .examingHeartbeatKey(examingSession.getExamRecordDataId());
        ExamingHeartbeat examingHeartbeat = redisClient.get(examingHeartbeatKey,
                ExamingHeartbeat.class);

        //秒
        long cost = null == examingHeartbeat ? 0 : examingHeartbeat.getCost();
        return examingSession.getExamDuration() <= cost * 1000;
    }

    /**
     * 是否超过断点续考时间
     *
     * @param examingSession
     * @return
     */
    private boolean isOverBreakpointTime(ExamingSession examingSession) {
        long now = System.currentTimeMillis();

        String examingActiveTimeKey = RedisKeyHelper.getBuilder()
                .examingActiveTimeKey(examingSession.getExamRecordDataId());
        ExamingActivityTime examingActiveTime = redisClient.get(examingActiveTimeKey,
                ExamingActivityTime.class);

        if (null == examingActiveTime) {
            this.debugLog("validating isOverBreakpointTime... and examingActiveTime is null...", examingSession.getExamRecordDataId());
        } else {
            this.debugLog("validating isOverBreakpointTime... and the json of examingActiveTime is..." + JsonUtil.toJson(examingActiveTime)
                    , examingSession.getExamRecordDataId());
        }

        long activeTime = null == examingActiveTime
                ? System.currentTimeMillis()
                : examingActiveTime.getActiveTime();
        return now - activeTime >= examingSession.getExamReconnectTime().intValue() * 60 * 1000;
    }

    private void debugLog(String msg, Long examRecordDataId) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[HAND-IN-EXAM-EXECUTOR-" + examRecordDataId + "]：" + msg);
        }
    }

    private void errorLog(String msg, Exception e, Long examRecordDataId) {
        if (LOG.isErrorEnabled()) {
            LOG.error("[HAND-IN-EXAM-EXECUTOR-" + examRecordDataId + "]：" + msg, e);
        }
    }

}
