package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreNoticeQueueCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.SyncExamDataCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamCaptureBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamSyncCaptureBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.AddExamScoreNoticeQueueReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.SyncExamDataReq;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdatePartialExamRecordReq;
import cn.com.qmth.examcloud.core.oe.task.service.CommonService;
import cn.com.qmth.examcloud.core.oe.task.service.ExamBossService;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import cn.com.qmth.examcloud.support.examing.ExamBoss;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.support.helper.FaceBiopsyHelper;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description 同步考试数据执行器
 * @Author lideyin
 * @Date 2019/12/19 16:22
 * @Version 1.0
 */
@Deprecated
@Component
public class SyncExamDataExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    private static final Logger LOG = LoggerFactory.getLogger(SyncExamDataExecutor.class);

    @Autowired
    private SyncExamDataCloudService syncExamDataCloudService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private ExamRecordDataCloudService examRecordDataCloudService;

    @Autowired
    private ExamScoreNoticeQueueCloudService examScoreNoticeQueueCloudService;

    @Autowired
    private ExamBossService examBossService;

    @Autowired
    private CommonService commonService;

    @Override
    public void execute(Long key, ExamRecordData examRecordData,
                        List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {

        Long studentId = examRecordData.getStudentId();
        String sequenceLockKey = Constants.EXAM_CONTROL_LOCK_PREFIX + studentId;
        long st = System.currentTimeMillis();

        try {
            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            //如果已同步，直接交给下一节点
            if (SyncStatus.SYNCED == examRecordData.getSyncStatus()) {
                outList.add(new KeyValuePair<>(key, examRecordData));

                LOG.info("syncExamData already synced, tempExamRecordDataId = {}, examStudentId = {}",
                        examRecordData.getId(), examRecordData.getExamStudentId());
                return;
            }

            //处理上一节点中，指定时间内仍未处理完成的数据
            Long examRecordDataId = examRecordData.getId();
            if (ExamRecordStatus.EXAM_HAND_IN == examRecordData.getExamRecordStatus() ||
                    ExamRecordStatus.EXAM_AUTO_HAND_IN == examRecordData.getExamRecordStatus()) {

                this.debugLog("prepare to execute last node's hand in data...", examRecordDataId);
                long startTime = System.currentTimeMillis();
                examRecordData = examRecordDataService.processAfterHandInExam(examRecordDataId);

                this.debugLog("T001.execute last node's hand in data success...耗时：" +
                        (System.currentTimeMillis() - startTime) + " ms", examRecordDataId);
            }

            //如果考试记录状态为完结状态，则开始同步数据到正式库
            if (ExamRecordStatus.EXAM_END == examRecordData.getExamRecordStatus() ||
                    ExamRecordStatus.EXAM_OVERDUE == examRecordData.getExamRecordStatus()) {

                this.debugLog("prepare to sync data...", examRecordDataId);

                //同步数据
                SyncExamDataReq syncReq = new SyncExamDataReq();

                syncReq.setExamRecordPaperStruct(commonService.getExamRecordPaperStruct(examRecordDataId));

                syncReq.setExamRecordQuestions(commonService.getExamRecordQuestions(examRecordDataId));

                syncReq.setExamContinuedRecords(commonService.getExamContinuedRecords(examRecordDataId));

                syncReq.setExamProcessRecords(commonService.getExamProcessRecords(examRecordDataId));

                //开启人脸检测相关数据赋值
                Long rootOrgId = examRecordData.getRootOrgId();
                Long examId = examRecordData.getExamId();
                if (FaceBiopsyHelper.isFaceEnable(rootOrgId, examId, studentId)) {
                    List<ExamCaptureBean> captures = commonService.getExamCaptures(examRecordDataId);
                    syncReq.setExamCaptures(captures);

                    ExamSyncCaptureBean syncCapture = commonService.getExamSyncCapture(examRecordDataId);
                    syncReq.setExamSyncCapture(syncCapture);

                    //虚拟摄像头进入待审核，且有虚拟摄像头的或者同步没有照片的，更新缓存
                    boolean hasVirtualCamera = captures.stream().anyMatch(e -> e.getHasVirtualCamera() != null && e.getHasVirtualCamera());
                    if (examRecordDataService.isVirtualToWaiting(examId) && hasVirtualCamera || syncCapture == null) {
                        examRecordData.setIsWarn(true);
                        examRecordDataService.saveExamRecordDataCache(examRecordDataId, examRecordData);
                    }

                    if (FaceBiopsyHelper.isFaceVerify(rootOrgId, examId, studentId)) {
                        syncReq.setExamFaceLivenessVerifies(commonService.getExamFaceLivenessVerifies(examRecordDataId));

                        syncReq.setFaceBiopsy(commonService.getFaceBiopsy(examRecordDataId));

                        syncReq.setFaceLiveVerifyRecords(commonService.getFaceLiveVerifyRecords(examRecordDataId));
                    }
                }

                syncReq.setExamRecordData(commonService.copyExamRecordDataFrom(examRecordData));

                long startTime = System.currentTimeMillis();
                //同步数据
                syncExamDataCloudService.syncExamData(syncReq);
                LOG.info("syncExamData ok, tempExamRecordDataId = {}, examStudentId = {}, cost {} ms",
                        examRecordDataId, examRecordData.getExamStudentId(), System.currentTimeMillis() - startTime);

                //考试完结次数加1
                ExamBoss examBoss = examBossService.getExamBoss(examRecordData.getExamStudentId());
                if (null != examBoss) {
                    examBoss.setEndCount(examBoss.getEndCount() + 1);

                    startTime = System.currentTimeMillis();

                    examBossService.saveExamBoss(examRecordData.getExamStudentId(), examBoss);

                    this.debugLog("T003.考试完结次数加1,并saveExamBoss耗时..." +
                            (System.currentTimeMillis() - startTime) + " ms", examRecordDataId);
                }

                //设置并保存考试记录的同步状态
                startTime = System.currentTimeMillis();

                examRecordData.setSyncStatus(SyncStatus.SYNCED);
                setAndSaveExamRecordDataSyncStatus(examRecordDataId);
                this.debugLog("T004.设置并保存考试记录的同步状态,耗时..." +
                        (System.currentTimeMillis() - startTime) + " ms", examRecordDataId);

                //保存成绩通知队列
                String isPushScore = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                        examRecordData.getStudentId(), ExamProperties.PUSH_SCORE.name()).getValue();
                if (Constants.isTrue.equals(isPushScore)) {
                    AddExamScoreNoticeQueueReq req = new AddExamScoreNoticeQueueReq();
                    req.setRootOrgId(rootOrgId);
                    examScoreNoticeQueueCloudService.addExamScoreNoticeQueue(req);
                }

                outList.add(new KeyValuePair<>(key, examRecordData));

                this.debugLog("sync data and change status success...", examRecordDataId);

                return;
            }

            //其它状态的数据，直接交给下一个节点处理
            outList.add(new KeyValuePair<>(key, examRecordData));

            this.debugLog("current status is '" + examRecordData.getExamRecordStatus().name() +
                    "'.do nothing and go to the next node...", examRecordData.getId());
        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
            this.debugLog("T999 同步数据合计耗时:" + (System.currentTimeMillis() - st) + " ms", examRecordData.getId());
        }
    }

    /**
     * 设置并保存考试记录的同步状态
     *
     * @param examRecordDataId
     */
    private void setAndSaveExamRecordDataSyncStatus(Long examRecordDataId) {
        ExamRecordData examRecordData = examRecordDataService.getExamRecordDataCache(examRecordDataId);

        if (SyncStatus.SYNCED == examRecordData.getSyncStatus()) {
            return;
        }

        examRecordData.setSyncStatus(SyncStatus.SYNCED);
        examRecordDataService.saveExamRecordDataCache(examRecordDataId, examRecordData);

        try {
            //同步更新数据库表中的同步状态
            UpdatePartialExamRecordReq uperReq = new UpdatePartialExamRecordReq(examRecordDataId);
            uperReq.setSyncStatus(SyncStatus.SYNCED.name());
            examRecordDataCloudService.updatePartialExamRecord(uperReq);
        } catch (Exception e) {
            this.errorLog("更新数据库表中的同步状态失败", examRecordDataId, e);
        }
    }

    private void debugLog(String msg, Long examRecordDataId) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[SYNC-EXAM-DATA-EXECUTOR-" + examRecordDataId + "]：" + msg);
        }
    }

    private void errorLog(String msg, Long examRecordDataId, Exception e) {
        if (LOG.isErrorEnabled()) {
            LOG.error("[SYNC-EXAM-DATA-EXECUTOR-" + examRecordDataId + "]：" + msg, e);
        }
    }

}
