package cn.com.qmth.examcloud.core.oe.task.starter.config;

import cn.com.qmth.examcloud.commons.helpers.concurrency.simple.ConcurrentTask;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamCaptureQueueRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamCaptureQueueEntity;
import cn.com.qmth.examcloud.core.oe.task.service.bean.ExamCaptureQueueInfo;
import cn.com.qmth.examcloud.core.oe.task.service.worker.BaiduFaceLivenessWorker;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import org.apache.logging.log4j.ThreadContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 启动百度活体检测任务
 */
@Deprecated
@Component
@Order(201)
public class ProcessBaiduFacelivenessTask implements ApplicationRunner {

    private static final Logger LOG = LoggerFactory.getLogger(ProcessBaiduFacelivenessTask.class);

    @Autowired
    ExamCaptureQueueRepo examCaptureQueueRepo;

    @Value("${examcloud.face.compare.task.enable}")
    private Boolean faceCompareTaskEnable;

    private int process(ConcurrentTask<ExamCaptureQueueInfo> concurrentTask,
                        String processBatchNum, Integer limit) {

        // 如果队列没满，则从数据库中查数据并插入
        List<ExamCaptureQueueEntity> examCaptureQueueList = examCaptureQueueRepo
                .findNeedFacelivenessDetectExamCaptureQueuesLimit(limit, processBatchNum);

        if (null == examCaptureQueueList || examCaptureQueueList.isEmpty()) {
            LOG
                    .debug("[PROCESS_BAIDUFACELIVENESS." + processBatchNum + "] 抓拍队列中没有取到数据，2秒后重试");
            return 0;
        }

        LOG.debug("[PROCESS_BAIDUFACELIVENESS." + processBatchNum + "] 抓拍队列中的数据条数为："
                + examCaptureQueueList.size());

        for (ExamCaptureQueueEntity offeredQueueEntity : examCaptureQueueList) {
            while (true) {
                try {
                    ThreadContext.put("TRACE_ID", "Q_" + offeredQueueEntity.getId());
                    ThreadContext.put("CALLER", "BAIDU_TASK");

                    examCaptureQueueRepo.updateProcessBatchNum(offeredQueueEntity.getId(), processBatchNum);
                    boolean offerSuccess = concurrentTask.offerElement(copyExamCaptureQueueInfoFrom(offeredQueueEntity));
                    if (offerSuccess) {
                        LOG.debug("[PROCESS_BAIDUFACELIVENESS." + processBatchNum
                                + "] 向工作队列中添加数据成功：fileUrl=" + offeredQueueEntity.getFileUrl());
                        break;
                    }

                    LOG.debug("[PROCESS_BAIDUFACELIVENESS." + processBatchNum
                            + "] 向工作队列中添加数据失败，30秒后重试：fileUrl=" + offeredQueueEntity.getFileUrl());

                    Util.sleep(PropertyHolder.getInt("$capture.queue.offer.sleepSeconds.", 30));
                } finally {
                    ThreadContext.clearAll();
                }

            }
        }
        return examCaptureQueueList.size();
    }

    @Override
    public void run(ApplicationArguments args) {
        faceCompareTaskEnable = false;

        LOG.warn("ProcessBaiduFacelivenessTask enable is " + faceCompareTaskEnable);
        if (!faceCompareTaskEnable) {
            return;
        }

        ConcurrentTask<ExamCaptureQueueInfo> concurrentTask = new ConcurrentTask<ExamCaptureQueueInfo>("百度活体检测");
        concurrentTask.setMaxActiveThreadSize(
                PropertyHolder.getInt("$capture.thread.maxActiveThreadSize", 100));
        concurrentTask.setMinThreadSize(PropertyHolder.getInt("$capture.thread.minThreadSize", 2));
        concurrentTask.setWorker(new BaiduFaceLivenessWorker());
        concurrentTask.setMaxActiveThreadSize(PropertyHolder.getInt("$capture.baidu.thread.maxActiveThreadSize", 30));
        concurrentTask.start();
        // 当前获取数据的批次号(默认用时间戳)
        String processBatchNum = "B_" + System.currentTimeMillis();

        LOG.debug("[PROCESS_BAIDUFACELIVENESS." + processBatchNum + "] 启动百度处理服务...");

        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                while (true) {
                    try {
                        Integer limit = PropertyHolder.getInt("$capture.queue.limit", 100);
                        for (int i = 0; i < 5; i++) {
                            int realCount = process(concurrentTask, processBatchNum, limit);
                            if (realCount < limit) {
                                break;
                            }
                        }
                        Util.sleep(PropertyHolder.getInt("$capture.queue.read.sleepSeconds.", 2));
                    } catch (Exception e) {
                        LOG.error("[PROCESS_FACEPP." + processBatchNum + "] 百度活体检测出异常,3秒后重试",
                                e);
                        Util.sleep(3);
                    }
                }

            }
        });

        thread.setDaemon(true);
        thread.start();
    }

    private ExamCaptureQueueInfo copyExamCaptureQueueInfoFrom(ExamCaptureQueueEntity entity) {
        ExamCaptureQueueInfo info = new ExamCaptureQueueInfo();
        info.setFaceCompareResult(entity.getFaceCompareResult());
        info.setFacelivenessResult(entity.getFacelivenessResult());
        info.setPass(entity.getIsPass());
        info.setStranger(entity.getIsStranger());
        info.setFaceCompareStartTime(entity.getFaceCompareStartTime());
        info.setStatus(entity.getStatus());
        info.setBaseFaceToken(entity.getBaseFaceToken());
        info.setCameraInfos(entity.getCameraInfos());
        info.setCreationTime(entity.getCreationTime());
        info.setErrorMsg(entity.getErrorMsg());
        info.setErrorNum(entity.getErrorNum());
        info.setExamRecordDataId(entity.getExamRecordDataId());
        info.setExtMsg(entity.getExtMsg());
        info.setFileName(entity.getFileName());
        info.setFileUrl(entity.getFileUrl());
        info.setHasVirtualCamera(entity.getHasVirtualCamera());
        info.setId(entity.getId());
        info.setPriority(entity.getPriority());
        info.setProcessBatchNum(entity.getProcessBatchNum());
        info.setStudentId(entity.getStudentId());
        info.setUpdateTime(entity.getUpdateTime());
        return info;
    }

}
