package cn.com.qmth.examcloud.core.oe.task.starter.config;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.core.oe.task.service.worker.FaceVerifyCounter;
import cn.com.qmth.examcloud.exchange.inner.api.SmsCloudService;
import cn.com.qmth.examcloud.exchange.inner.api.request.SendSmsReq;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import com.google.common.collect.Maps;
import com.googlecode.aviator.AviatorEvaluator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description 人脸比对预警任务
 * @Author lideyin
 * @Date 2019/9/12 10:52
 * @Version 1.0
 */
@Deprecated
@Component
@Order(202)
public class ProcessFaceCompareAlarmTask implements ApplicationRunner {

    @Autowired
    SmsCloudService smsCloudService;

    @Value("${examcloud.face.compare.task.enable}")
    private Boolean faceCompareTaskEnable;

    private static final Logger LOG = LoggerFactory.getLogger(ProcessFaceCompareAlarmTask.class);

    @Override
    public void run(ApplicationArguments args) throws Exception {
        faceCompareTaskEnable = false;

        LOG.warn("ProcessFaceCompareAlarmTask enable is " + faceCompareTaskEnable);
        if (!faceCompareTaskEnable) {
            return;
        }

        Thread thread = new Thread(new Runnable() {

            @Override
            public void run() {
                while (true) {
                    try {
                        faceCompareAlarm();

                    } catch (Exception e) {
                        LOG.error("[FACE_COMPARE_ALARM.] 人脸比对预警出现异常 ", e);
                    }
                    // 每分钟轮循一次
                    Util.sleep(60);
                }
            }
        });

        thread.setDaemon(true);
        thread.start();
    }

    /**
     * 人脸比对，如果有必要则发短信
     */
    private void faceCompareAlarm() {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[FACE_COMPARE_ALARM] 进入人脸" + System.currentTimeMillis()
                    + "....totalCount="
                    + FaceVerifyCounter.getFaceCompareCount() + " ,failCount="
                    + FaceVerifyCounter.getFaceCompareFailedCount());
        }

        // 默认每分钟失败率超过50%则发短信报警，且总数不少于10次则短信报警
        if (needSmsAlarm()) {
            SysPropertyCacheBean faceCompareSmsAssemblyCodeProperty = CacheHelper
                    .getSysProperty("capture.faceCompare.smsAssemblyCode");

            if (!faceCompareSmsAssemblyCodeProperty.getHasValue()) {
                LOG.error("[FACE_COMPARE_ALARM.] 未配置人脸比对的短信模板代码，totalCount="
                        + FaceVerifyCounter.getFaceCompareCount()
                        + ",errorCount="
                        + FaceVerifyCounter.getFaceCompareFailedCount());
                throw new StatusException("300001", "未配置人脸比对的短信模板代码");
            }
            SysPropertyCacheBean smsPhoneProperty = CacheHelper
                    .getSysProperty("capture.sms.phones");
            if (!smsPhoneProperty.getHasValue()) {
                if (LOG.isErrorEnabled()) {
                    LOG.error("[FACE_COMPARE_ALARM.] 未配置图片处理失败的通知手机号，totalCount="
                            + FaceVerifyCounter.getFaceCompareCount()
                            + ",errorCount="
                            + FaceVerifyCounter.getFaceCompareFailedCount());
                }
                throw new StatusException("300002", "未配置图片处理失败的通知手机号");
            }

            List<String> phoneList = Arrays
                    .asList(smsPhoneProperty.getValue().toString().split(","));

            // List<String> phoneList = Arrays.asList("13717595977");
            SendSmsReq sendSmsReq = new SendSmsReq();
            sendSmsReq.setPhoneList(phoneList);
            sendSmsReq.setSmsAssemblyCode(faceCompareSmsAssemblyCodeProperty.getValue().toString());
            // sendSmsReq.setSmsAssemblyCode("FACECOMPARE");

            HashMap<String, String> params = new HashMap<>();
            params.put("totalCount",
                    String.valueOf(FaceVerifyCounter.getFaceCompareCount()));
            params.put("errorCount", String
                    .valueOf(FaceVerifyCounter.getFaceCompareFailedCount()));
            sendSmsReq.setParams(params);
            try {
                if (LOG.isDebugEnabled()) {
                    LOG.debug("[FACE_COMPARE_ALARM.] 开始调用发送短信接口，totalCount="
                            + FaceVerifyCounter.getFaceCompareCount()
                            + ",errorCount="
                            + FaceVerifyCounter.getFaceCompareFailedCount());
                }
                smsCloudService.sendSms(sendSmsReq);
            } catch (Exception e) {
                LOG.error("[FACE_COMPARE_ALARM.] 发送短信出现异常", e);
            }
        }
        // 每1分钟重置一次总数量与失败数量
        FaceVerifyCounter.resetAllFaceCompareCount();
    }

    /**
     * 是否需要短信报警
     *
     * @return boolean
     */
    private boolean needSmsAlarm() {
        if (FaceVerifyCounter.getFaceCompareCount() == 0) {
            return false;
        }
        SysPropertyCacheBean expressionProperty = CacheHelper.getSysProperty("capture.faceCompare.expression.alarm");
        if (expressionProperty.getHasValue()) {
            String expression = expressionProperty.getValue().toString();
            Map<String, Object> env = Maps.newHashMap();
            env.put("totalCount", FaceVerifyCounter.getFaceCompareCount());
            env.put("failedCount", FaceVerifyCounter.getFaceCompareFailedCount());
            try {
                return (Boolean) AviatorEvaluator.execute(expression, env, true);
            } catch (Exception e) {
                throw new StatusException("300004", "failed to execute expression. expression=" + expression);
            }

        } else {
            // 默认每分钟失败率超过50%则发短信报警，且失败总数不少于10次则短信报警
            return FaceVerifyCounter.getFaceCompareFailedCount() > 10 &&
                    FaceVerifyCounter.getFaceLivenessDetectFailurePercent() > 50;
        }
    }

}
